@@ -33,8 +33,8 @@ def handle_sigterm(sig, frame):
 
 
 def run_flent(gui=False):
-    if sys.version_info[:2] < (3, 5):
-        sys.stderr.write("Sorry, Flent requires v3.5 or later of Python.\n")
+    if sys.version_info[:2] < (3, 6):
+        sys.stderr.write("Sorry, Flent requires v3.6 or later of Python.\n")
         sys.exit(1)
     try:
         try:
@@ -47,7 +47,7 @@ def run_flent(gui=False):
 
         setup_console()
         logger = get_logger(__name__)
-        logger.debug("Flent executed as %s", sys.argv)
+        logger.debug("Flent executed as %s in PID %d", sys.argv, os.getpid())
 
         try:
             signal.signal(signal.SIGTERM, handle_sigterm)
