@@ -35,10 +35,8 @@ except ImportError:
     import pickle
 
 from argparse import SUPPRESS
-from datetime import datetime
 from itertools import chain
 from multiprocessing import Pool, Queue
-from distutils.version import LooseVersion
 
 from flent import util, batch, loggers, resultset, plotters
 from flent.build_info import DATA_DIR, VERSION
@@ -47,16 +45,16 @@ from flent.loggers import get_logger, add_log_handler, remove_log_handler, \
 from flent.resultset import ResultSet
 from flent.settings import ListTests, new as new_settings, plot_group
 
+try:
+    import matplotlib
+    matplotlib.use("Agg")
+except ImportError as e:
+    raise RuntimeError(f"The GUI requires matplotlib, please ensure it's installed ({e}).")
+
 logger = get_logger(__name__)
 
 mswindows = (sys.platform == "win32")
 
-# Python 2/3 compatibility
-try:
-    unicode
-except NameError:
-    unicode = str
-
 try:
     from os import cpu_count
 except ImportError:
@@ -67,74 +65,34 @@ try:
 except NotImplementedError:
     CPU_COUNT = 1
 
-if plotters.MPL_VER < LooseVersion("1.4.2"):
-    raise RuntimeError("The GUI requires matplotlib version >= 1.4.2 to work.")
-else:
-    import matplotlib
-    matplotlib.use("Agg")
 
-FORCE_QT4 = bool(os.getenv("FORCE_QT4", False))
 try:
-    if FORCE_QT4:
-        raise ImportError("Force fallback to Qt4")
+    import qtpy
+    from qtpy import QtCore, QtGui, uic
 
-    from PyQt5 import QtCore, QtGui, uic
-
-    from PyQt5.QtWidgets import QMessageBox, QFileDialog, QTreeView, \
+    from qtpy.QtWidgets import QMessageBox, QFileDialog, QTreeView, \
         QAbstractItemView, QMenu, QAction, QTableView, QHeaderView, \
         QFormLayout, QHBoxLayout, QVBoxLayout, QApplication, QPlainTextEdit, \
         QWidget, QLineEdit, QComboBox, QSpinBox, QDoubleSpinBox, QScrollArea, \
-        QPushButton, QShortcut
+        QPushButton, QShortcut, QMainWindow, QDialog
 
-    from PyQt5.QtGui import QFont, QCursor, QMouseEvent, QKeySequence, \
+    from qtpy.QtGui import QFont, QCursor, QMouseEvent, QKeySequence, \
         QResizeEvent, QDesktopServices, QValidator, QGuiApplication
 
-    get_clipboard = QGuiApplication.clipboard
-
-    from PyQt5.QtCore import Qt, QIODevice, QByteArray, \
-        QDataStream, QSettings, QTimer, QEvent, pyqtSignal, \
+    from qtpy.QtCore import Qt, QIODevice, QByteArray, \
+        QDataStream, QSettings, QTimer, QEvent, Signal, \
         QAbstractItemModel, QAbstractTableModel, QModelIndex, \
         QItemSelectionModel, QStringListModel, QUrl
 
-    from PyQt5.QtNetwork import QLocalSocket, QLocalServer
+    from qtpy.QtNetwork import QLocalSocket, QLocalServer
 
     from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg \
         as FigureCanvas
     from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT \
         as NavigationToolbar
-    QTVER = 5
-except ImportError:
-    try:
-        from PyQt4 import QtCore, QtGui, uic
 
-        from PyQt4.QtGui import QMessageBox, QFileDialog, QTreeView, \
-            QAbstractItemView, QMenu, QAction, QFont, QTableView, QCursor, \
-            QHeaderView, QFormLayout, QHBoxLayout, QVBoxLayout, \
-            QItemSelectionModel, QMouseEvent, QApplication, QStringListModel, \
-            QKeySequence, QResizeEvent, QPlainTextEdit, QDesktopServices, \
-            QWidget, QLineEdit, QComboBox, QSpinBox, QDoubleSpinBox, QScrollArea,\
-            QPushButton, QValidator, QShortcut
-
-        get_clipboard = QApplication.clipboard
-
-        from PyQt4.QtCore import Qt, QIODevice, QByteArray, \
-            QDataStream, QSettings, QTimer, QEvent, pyqtSignal, \
-            QAbstractItemModel, QAbstractTableModel, QModelIndex, QUrl
-
-        from PyQt4.QtNetwork import QLocalSocket, QLocalServer
-
-        from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg \
-            as FigureCanvas
-        from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT \
-            as NavigationToolbar
-
-        QTVER = 4
-
-        if not FORCE_QT4:
-            logger.warning("Falling back to Qt4 for the GUI. "
-                           "Please consider installing PyQt5.\n")
-    except ImportError:
-        raise RuntimeError("Unable to find a usable PyQt version.")
+except ImportError as e:
+    raise RuntimeError(f"Unable to find a usable Qt version, please ensure qtpy is installed ({e}).")
 
 
 # The file selector dialog on OSX is buggy, so switching allowed file extensions
@@ -186,9 +144,9 @@ def run_gui(settings, test_mode=False):
     # Start up the Qt application and exit when it does
     app = QApplication(sys.argv[:1])
     mainwindow = MainWindow(settings)
-    mainwindow.show()
     if test_mode:
-        mainwindow.defer_close()
+        mainwindow.defer_close(app)
+    mainwindow.show()
     return app.exec_()
 
 
@@ -250,24 +208,8 @@ def check_running(settings):
     return False
 
 
-def get_ui_class(filename):
-    """Helper method to dynamically load a .ui file, construct a class
-    inheriting from the ui class and the associated base class, and return
-    that constructed class.
-
-    This allows subclasses to inherit from the output of this function."""
-
-    try:
-        ui, base = uic.loadUiType(os.path.join(DATA_DIR, 'ui', filename))
-    except Exception as e:
-        raise RuntimeError("While loading ui file '%s': %s" % (filename, e))
-
-    class C(ui, base):
-
-        def __init__(self, *args):
-            base.__init__(self, *args)
-            self.setupUi(self)
-    return C
+def get_ui_file(filename):
+    return os.path.join(DATA_DIR, 'ui', filename)
 
 
 class LoadedResultset(dict):
@@ -292,10 +234,11 @@ def results_load_helper(filename):
         return None
 
 
-class MainWindow(get_ui_class("mainwindow.ui")):
+class MainWindow(QMainWindow):
 
     def __init__(self, settings):
         super(MainWindow, self).__init__()
+        uic.loadUi(get_ui_file("mainwindow.ui"), self)
         self.settings = settings
         self.last_dir = os.getcwd()
 
@@ -334,6 +277,10 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         self.viewArea.tabCloseRequested.connect(self.close_tab)
         self.viewArea.currentChanged.connect(self.activate_tab)
 
+        self.openFileButton.clicked.connect(self.on_open)
+        self.runTestButton.clicked.connect(self.run_test)
+        self.update_empty_view()
+
         self.plotDock.visibilityChanged.connect(self.plot_visibility)
         self.metadataDock.visibilityChanged.connect(self.metadata_visibility)
         self.plotSettingsDock.visibilityChanged.connect(self.plot_settings_visibility)
@@ -404,7 +351,8 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         QShortcut(QKeySequence("Ctrl+Up"),
                   self).activated.connect(self.prev_plot)
 
-        logger.info("GUI loaded. Running on PyQt v%s.", QtCore.PYQT_VERSION_STR)
+        logger.info("GUI loaded. Using Qt through %s v%s.", qtpy.API,
+                    QtCore.__version__)
 
     def read_log_queue(self):
         while not self.log_queue.empty():
@@ -464,6 +412,14 @@ class MainWindow(get_ui_class("mainwindow.ui")):
             self.openFilesView.horizontalHeader().restoreState(value)
             self.openFilesView.setSortingEnabled(True)
 
+    def update_empty_view(self):
+        if self.viewArea.count():
+            self.viewArea.show()
+            self.emptyWidget.hide()
+        else:
+            self.viewArea.hide()
+            self.emptyWidget.show()
+
     def closeEvent(self, event):
         # Cleaning up matplotlib figures can take a long time; disable it when
         # the application is exiting.
@@ -485,7 +441,7 @@ class MainWindow(get_ui_class("mainwindow.ui")):
 
     def keyPressEvent(self, event):
         widget = self.viewArea.currentWidget()
-        text = unicode(event.text())
+        text = str(event.text())
         if widget and text in ('x', 'X', 'y', 'Y'):
             a = text.lower()
             d = 'in' if a == text else 'out'
@@ -561,7 +517,7 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         if isinstance(filenames, tuple):
             filenames = filenames[0]
         if filenames:
-            self.last_dir = os.path.dirname(unicode(filenames[0]))
+            self.last_dir = os.path.dirname(str(filenames[0]))
 
         return filenames
 
@@ -648,9 +604,9 @@ class MainWindow(get_ui_class("mainwindow.ui")):
             self.load_files(self.defer_load)
             self.defer_load = None
 
-    def defer_close(self):
+    def defer_close(self, qapp):
         self.close_timer = QTimer(self)
-        self.close_timer.timeout.connect(self.close)
+        self.close_timer.timeout.connect(qapp.quit, Qt.QueuedConnection)
         self.close_timer.start()
 
     def shorten_titles(self, titles):
@@ -709,6 +665,7 @@ class MainWindow(get_ui_class("mainwindow.ui")):
             widget.setParent(None)
             widget.deleteLater()
             self.shorten_tabs()
+        self.update_empty_view()
         self.busy_end()
 
     def close_all(self):
@@ -717,6 +674,7 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         for i in range(self.viewArea.count()):
             widgets.append(self.viewArea.widget(i))
         self.viewArea.clear()
+        self.update_empty_view()
         for w in widgets:
             w.setUpdatesEnabled(False)
             w.disconnect_all()
@@ -830,6 +788,7 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         if title is None:
             title = widget.title
         idx = self.viewArea.addTab(widget, title)
+        self.update_empty_view()
         if hasattr(widget, "long_title"):
             self.viewArea.setTabToolTip(idx, widget.long_title)
         if focus or self.focus_new:
@@ -851,7 +810,7 @@ class MainWindow(get_ui_class("mainwindow.ui")):
             titles = self.shorten_titles([r.title for r in results])
         else:
             results = list(filter(None, self.worker_pool.map(results_load_helper,
-                                                             map(unicode,
+                                                             map(str,
                                                                  filenames))))
 
             titles = self.shorten_titles([r['title'] for r in results])
@@ -862,7 +821,7 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         self.load_timer.start()
 
         if set_last_dir:
-            self.last_dir = os.path.dirname(unicode(filenames[-1]))
+            self.last_dir = os.path.dirname(str(filenames[-1]))
 
     def load_one(self):
         if not self.load_queue:
@@ -931,24 +890,31 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         dlg.exec_()
 
 
-class AboutDialog(get_ui_class("aboutdialog.ui")):
+class AboutDialog(QDialog):
 
     def __init__(self, parent):
         super(AboutDialog, self).__init__(parent)
+        uic.loadUi(get_ui_file("aboutdialog.ui"), self)
+
         self.aboutText.setText(ABOUT_TEXT.format(version=VERSION))
 
 
-class NewTestDialog(get_ui_class("newtestdialog.ui")):
+class NewTestDialog(QDialog):
 
     def __init__(self, parent, settings, log_queue):
         super(NewTestDialog, self).__init__(parent)
-        self.settings = settings.copy()
-        self.settings.INPUT = []
-        self.settings.GUI = False
+        uic.loadUi(get_ui_file("newtestdialog.ui"), self)
+        self.orig_settings = settings.copy()
+        self.orig_settings.INPUT = []
+        self.orig_settings.GUI = False
+        self.settings = self.orig_settings.copy()
         self.log_queue = log_queue
         self.pid = None
         self.aborted = False
 
+        if self.settings.NAME is None:
+            self.settings.NAME = "rrul"
+
         tests = ListTests.get_tests(settings)
         max_len = max([len(t[0]) for t in tests])
         for t, desc in tests:
@@ -1015,19 +981,19 @@ class NewTestDialog(get_ui_class("newtestdialog.ui")):
             logger.error("Output directory does not exist.")
             return
 
-        test = unicode(test)
-        host = unicode(host)
-        path = unicode(path)
+        test = str(test)
+        host = str(host)
+        path = str(path)
 
-        self.settings.HOSTS = [host]
+        self.settings.HOSTS = util.token_split(host)
         self.settings.NAME = test
-        self.settings.TITLE = unicode(self.testTitle.text())
+        self.settings.TITLE = str(self.testTitle.text())
         self.settings.LENGTH = self.testLength.value()
         self.settings.DATA_DIR = path
         self.settings.EXTENDED_METADATA = self.extendedMetadata.isChecked()
         self.settings.load_test(informational=True)
         self.settings.FORMATTER = "null"
-        self.settings.TIME = datetime.utcnow()
+        self.settings.TIME = util.utcnow()
 
         self.settings.DATA_FILENAME = None
         res = resultset.new(self.settings)
@@ -1067,6 +1033,7 @@ class NewTestDialog(get_ui_class("newtestdialog.ui")):
         self.monitor_timer.stop()
         self.pid = None
         self.aborted = False
+        self.settings = self.orig_settings.copy()
 
     def keyPressEvent(self, evt):
         if evt.key() == Qt.Key_Escape:
@@ -1084,13 +1051,13 @@ class NewTestDialog(get_ui_class("newtestdialog.ui")):
         if (p, s) == (0, 0):
             if not self.aborted:
                 elapsed = time.time() - self.start_time
-                self.progressBar.setValue(100 * elapsed / self.total_time)
+                self.progressBar.setValue(int(100 * elapsed / self.total_time))
         else:
-            self.reset()
             fn = os.path.join(self.settings.DATA_DIR,
                               self.settings.DATA_FILENAME)
             if os.path.exists(fn):
                 self.parent().load_files([fn])
+            self.reset()
 
 
 class QPlainTextLogger(loggers.Handler):
@@ -1106,17 +1073,24 @@ class QPlainTextLogger(loggers.Handler):
         self.widget = widget or QPlainTextEdit(parent)
         self.widget.setFont(font)
         self.widget.setReadOnly(True)
+        self.widget.destroyed.connect(self.destroy_widget)
 
         self.statusbar = statusbar
         self.timeout = timeout
 
     def emit(self, record):
+        if self.widget is None:
+            return
+
         msg = self.format(record)
         self.widget.appendPlainText(msg)
 
         if self.statusbar:
             self.statusbar.showMessage(record.message, self.timeout)
 
+    def destroy_widget(self):
+        self.widget = None
+
     def write(self, p):
         pass
 
@@ -1197,7 +1171,7 @@ class MetadataModel(QAbstractItemModel):
         if idx.column() == 0:
             return item.name
         elif idx.column() == 1:
-            return unicode(item.value)
+            return str(item.value)
 
     def parent(self, idx):
         item = idx.internalPointer()
@@ -1275,7 +1249,7 @@ class MetadataView(QTreeView):
                                                    1,
                                                    idx.parent()),
                                 Qt.DisplayRole)
-        get_clipboard().setText(val)
+        QGuiApplication.clipboard().setText(val)
 
     def setModel(self, model):
         super(MetadataView, self).setModel(model)
@@ -1334,7 +1308,7 @@ class ActionWidget(object):
 
 class BooleanActionWidget(ActionWidget, QComboBox):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(BooleanActionWidget, self).__init__(*args, **kwargs)
@@ -1359,7 +1333,7 @@ class BooleanActionWidget(ActionWidget, QComboBox):
 
 class ChoicesActionWidget(ActionWidget, QComboBox):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(ChoicesActionWidget, self).__init__(*args, **kwargs)
@@ -1407,12 +1381,12 @@ class NoneSpinBoxMixin(object):
 
 
 class NoneDoubleSpinBox(NoneSpinBoxMixin, QDoubleSpinBox):
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
 
 class IntActionWidget(ActionWidget, NoneSpinBoxMixin, QSpinBox):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(IntActionWidget, self).__init__(*args, **kwargs)
@@ -1432,7 +1406,7 @@ class IntActionWidget(ActionWidget, NoneSpinBoxMixin, QSpinBox):
 
 class FloatActionWidget(ActionWidget, NoneSpinBoxMixin, QDoubleSpinBox):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(FloatActionWidget, self).__init__(*args, **kwargs)
@@ -1451,7 +1425,7 @@ class FloatActionWidget(ActionWidget, NoneSpinBoxMixin, QDoubleSpinBox):
 
 class PairActionWidget(ActionWidget, QWidget):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, parent, action, widget=QLineEdit, **kwargs):
         super(PairActionWidget, self).__init__(parent, action, **kwargs)
@@ -1483,7 +1457,7 @@ class PairActionWidget(ActionWidget, QWidget):
 
 class FloatPairActionWidget(PairActionWidget):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         kwargs["widget"] = NoneDoubleSpinBox
@@ -1511,7 +1485,7 @@ class FloatPairActionWidget(PairActionWidget):
 
 class TextActionWidget(ActionWidget, QLineEdit):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(TextActionWidget, self).__init__(*args, **kwargs)
@@ -1532,9 +1506,9 @@ class TextActionWidget(ActionWidget, QLineEdit):
 
 class AddRemoveWidget(QWidget):
 
-    add_pressed = pyqtSignal()
-    remove_pressed = pyqtSignal('QWidget')
-    value_changed = pyqtSignal()
+    add_pressed = Signal()
+    remove_pressed = Signal('QWidget')
+    value_changed = Signal()
 
     def __init__(self, parent, subwidget):
         super(AddRemoveWidget, self).__init__(parent)
@@ -1570,7 +1544,7 @@ class AddRemoveWidget(QWidget):
 
 class MultiValWidget(ActionWidget, QWidget):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         widget = kwargs.get("widget", TextActionWidget)
@@ -1646,7 +1620,7 @@ class MultiValWidget(ActionWidget, QWidget):
 
 class SettingsWidget(QScrollArea):
 
-    values_changed = pyqtSignal()
+    values_changed = Signal()
 
     _widget_type_map = {int: IntActionWidget,
                         float: FloatActionWidget,
@@ -1654,7 +1628,7 @@ class SettingsWidget(QScrollArea):
                         util.float_pair_noomit: FloatPairActionWidget,
                         util.comma_list: MultiValWidget,
                         util.keyval: PairActionWidget,
-                        unicode: TextActionWidget}
+                        str: TextActionWidget}
 
     def __init__(self, parent, options, settings, compact=False):
         super(SettingsWidget, self).__init__(parent)
@@ -1755,7 +1729,7 @@ class ResultsetStore(object):
 
         def get_key(itm):
             try:
-                return unicode(itm.meta(key))
+                return str(itm.meta(key))
             except KeyError:
                 return ''
         if only:
@@ -1910,7 +1884,7 @@ class OpenFilesModel(QAbstractTableModel):
 
     def get_metadata(self, idx, name):
         try:
-            return unicode(self.open_files[idx].meta(name))
+            return str(self.open_files[idx].meta(name))
         except KeyError:
             return None
 
@@ -2054,10 +2028,11 @@ class OpenFilesHeader(QHeaderView):
         event.accept()
 
 
-class AddColumnDialog(get_ui_class("addcolumn.ui")):
+class AddColumnDialog(QDialog):
 
     def __init__(self, parent, path=None):
         super(AddColumnDialog, self).__init__(parent)
+        uic.loadUi(get_ui_file("addcolumn.ui"), self)
 
         self.metadataPathEdit.textChanged.connect(self.update_name)
         self.columnNameEdit.textEdited.connect(self.name_entered)
@@ -2076,10 +2051,10 @@ class AddColumnDialog(get_ui_class("addcolumn.ui")):
         self.columnNameEdit.setText(parts[-1])
 
     def get_path(self):
-        return unicode(self.metadataPathEdit.text())
+        return str(self.metadataPathEdit.text())
 
     def get_name(self):
-        return unicode(self.columnNameEdit.text())
+        return str(self.columnNameEdit.text())
 
 
 class UpdateDisabler(object):
@@ -2104,17 +2079,18 @@ class FigureManager(matplotlib.backend_bases.FigureManagerBase):
         return self.widget.title
 
 
-class ResultWidget(get_ui_class("resultwidget.ui")):
+class ResultWidget(QWidget):
 
-    update_start = pyqtSignal()
-    update_end = pyqtSignal()
-    plot_changed = pyqtSignal('QString', 'QString')
-    new_plot = pyqtSignal()
-    name_changed = pyqtSignal()
+    update_start = Signal()
+    update_end = Signal()
+    plot_changed = Signal('QString', 'QString')
+    new_plot = Signal()
+    name_changed = Signal()
     default_title = "New tab"
 
     def __init__(self, parent, settings, worker_pool):
         super(ResultWidget, self).__init__(parent)
+        uic.loadUi(get_ui_file("resultwidget.ui"), self)
         self.results = None
         self.settings = settings.copy()
         self.dirty = True
@@ -2131,6 +2107,7 @@ class ResultWidget(get_ui_class("resultwidget.ui")):
         self.plotter = None
         self.canvas = None
         self.needs_resize = False
+        self.was_destroyed = False
 
         self.new_plot.connect(self.get_plotter)
         self.async_fig = None
@@ -2189,7 +2166,7 @@ class ResultWidget(get_ui_class("resultwidget.ui")):
         elif isinstance(results, ResultSet):
             self.results = results
         else:
-            self.results = ResultSet.load_file(unicode(results))
+            self.results = ResultSet.load_file(str(results))
             self.settings.compute_missing_results(self.results)
 
         if plot and plot in self.settings.PLOTS:
@@ -2220,6 +2197,7 @@ class ResultWidget(get_ui_class("resultwidget.ui")):
     def disconnect_all(self):
         for s in (self.update_start, self.update_end, self.plot_changed):
             s.disconnect()
+        self.was_destroyed = True
 
     def disable_cleanup(self):
         if self.plotter is not None:
@@ -2228,7 +2206,7 @@ class ResultWidget(get_ui_class("resultwidget.ui")):
     def load_files(self, filenames):
         added = 0
         for f in filenames:
-            if self.add_extra(ResultSet.load_file(unicode(f))):
+            if self.add_extra(ResultSet.load_file(str(f))):
                 self.update(False)
                 added += 1
         self.redraw()
@@ -2300,7 +2278,7 @@ class ResultWidget(get_ui_class("resultwidget.ui")):
             return
         if isinstance(plot_name, QModelIndex):
             plot_name = self.plotModel.name_of(plot_name)
-        plot_name = unicode(plot_name)
+        plot_name = str(plot_name)
         if plot_name != self.settings.PLOT and plot_name in self.settings.PLOTS:
             self.settings.PLOT = plot_name
             self.plotSelectionModel.setCurrentIndex(
@@ -2377,6 +2355,8 @@ class ResultWidget(get_ui_class("resultwidget.ui")):
         self.setCursor(Qt.WaitCursor)
 
     def recv_plot(self, fig):
+        if self.was_destroyed:
+            return
         self.new_plot.emit()
 
     def get_plotter(self):
